<?php

namespace Drupal\layout_builder_dynamic_tabs;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefault;

/**
 * Class defines the 'Dynamic tabs' layout.
 */
class DynamicTabsLayout extends LayoutDefault {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->pluginDefinition->setRegions(
      $this->buildRegions($this->getConfiguration()['tabs'])
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      'tabs' => [],
      'label' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);
    if ($this->inPreview) {
      $build['#attached']['library'] = ['layout_builder_dynamic_tabs/lb-preview'];
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['tabs'] = [
      '#prefix' => '<div id="tabs-add-more">',
      '#suffix' => '</div>',
      '#type' => 'table',
      '#header' => [$this->t('Tabs'), $this->t('Weight')],
      '#attributes' => [
        'id' => 'layout-tabs-form-table',
        'class' => ['lb-dyn-tabs__settings'],
      ],
      '#empty' => $this->t('No items.'),
      '#tableselect' => FALSE,
      '#attached' => ['library' => ['layout_builder_dynamic_tabs/lb-preview']],
    ];

    $tabs_initial = static::initialTabSet();
    $tabs = $form_state->getCompleteFormState()
      ->getValue(['layout_settings', 'tabs'], $this->configuration['tabs'] ?: $tabs_initial);

    foreach ($tabs as $delta => $tab) {
      $form['tabs'][$delta] = [
        '#tree' => TRUE,
        '#weight' => $tab['weight'] ?? 50,
        'detail' => [
          '#type' => 'container',
          'label' => [
            '#title_display' => 'invisible',
            '#type' => 'textfield',
            '#required' => TRUE,
            '#size' => 20,
            '#title' => $this->t('Label for tab %ix', ['%ix' => $delta + 1]),
            '#default_value' => $tab['detail']['label'],
          ],
          'region_name' => [
            '#type' => 'hidden',
            '#default_value' => $tab['detail']['region_name'],
          ],
          'remove' => [
            '#type' => 'submit',
            '#value' => $this->t('Remove'),
            '#submit' => [[static::class, 'removeTabSubmit']],
            '#element_validate' => [],
            '#attributes' => ['class' => ['remove-item']],
            '#ajax' => [
              'callback' => [static::class, 'updateForm'],
              'wrapper' => 'tabs-add-more',
              'effect' => 'fade',
              'method' => 'replaceWith',
            ],
          ],
        ],
        'weight' => [
          '#type' => 'weight',
          '#title' => $this->t('Weight for tab %ix', ['%ix' => $delta + 1]),
          '#title_display' => 'invisible',
          '#delta' => 50,
          '#default_value' => $tab['weight'] ?? 50,
          '#attributes' => ['class' => ['weight', 'draggable-weight']],
        ],
        '#attributes' => [
          'class' => ['draggable', 'js-form-wrapper'],
        ],
      ];
    }
    $draggable = [
      'action' => 'order',
      'relationship' => 'sibling',
      'group' => 'draggable-weight',
      'table_id' => 'layout-tabs-form-table',
    ];
    drupal_attach_tabledrag($form['tabs'], $draggable);
    uasort(
      $form['tabs'],
      ['Drupal\Component\Utility\SortArray', 'sortByWeightProperty']
    );

    $form['add_more'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another tab'),
      '#submit' => [[static::class, 'addMoreSubmit']],
      '#ajax' => [
        'callback' => [static::class, 'updateForm'],
        'wrapper' => 'tabs-add-more',
        'effect' => 'fade',
        'method' => 'replaceWith',
      ],
      '#element_validate' => [],
    ];

    $form['extra_wrapper_classes'] = [
      '#type' => 'textfield',
      '#size' => 30,
      '#title' => $this->t('Extra wrapper classes (optional)'),
      '#description' => $this->t('Custom CSS classnames for section container element.'),
      '#default_value' => $this->configuration['extra_wrapper_classes'],
    ];

    return $form;
  }

  /**
   * Submission handler for the "Add another tab" button.
   */
  public static function addMoreSubmit(array $form, FormStateInterface $form_state) {
    $tabs = $form_state->getValue(['layout_settings', 'tabs'], []);
    $count = count($tabs) + 1;
    $tabs[] = [
      'detail' => [
        'label' => 'Tab ' . $count,
        'region_name' => 'tab_' . $count,
      ],
      'weight' => $count,
    ];
    $form_state->setValue(['layout_settings', 'tabs'], $tabs);
    $form_state->setRebuild();
  }

  /**
   * Submission handler for the "Remove tab" button.
   */
  public static function removeTabSubmit(array $form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $tabs = $form_state->getValue(['layout_settings', 'tabs'], []);

    if (count($tabs) === 1) {
      $form_state->setValue(['layout_settings', 'tabs'], [self::initialTabSet()[0]]);
      $form_state->setRebuild();
      return;
    }

    $parents = $button['#parents'];
    // Remove the button.
    array_pop($parents);
    // Remove the detail container.
    array_pop($parents);
    unset($tabs[end($parents)]);

    $form_state->setValue(['layout_settings', 'tabs'], $tabs);
    $form_state->setRebuild();
  }

  /**
   * Ajax callback for the "Add another tab" button.
   */
  public static function updateForm(array $form, FormStateInterface $form_state) {
    return $form['layout_settings']['tabs'] ?? [];
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $tabs = $form_state->getValue(['tabs'], []);
    $labels = array_map(fn (array $item) => $item['detail']['label'], $tabs);
    if (count($labels) > count(array_unique($labels))) {
      $form_state->setErrorByName('layout_settings[tabs][0][detail][label]', $this->t('Each tab name must be unique'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['label'] = $form_state->getValue('label');
    $this->configuration['tabs'] = $form_state->getValue('tabs', []);
    $this->configuration['extra_wrapper_classes'] = $form_state->getValue('extra_wrapper_classes', '');
  }

  /**
   * Builds regions from tab configuration.
   *
   * @param array $tabs
   *   Tabs.
   *
   * @return array
   *   Regions.
   */
  protected function buildRegions(array $tabs): array {
    $regions = [];
    foreach (array_values($tabs) as $delta => $tab) {
      $regions['tab_' . ($delta + 1)] = [
        'label' => $tab['detail']['label'] ?? 'Tab ' . ($delta + 1),
      ];
    }
    return $regions;
  }

  /**
   * Get the initial tabs array value.
   *
   * @return array
   *   Initial tabs.
   */
  protected static function initialTabSet(): array {
    return [
      ['detail' => ['label' => 'Tab 1', 'region_name' => 'tab_1'], 'weight' => 1],
      ['detail' => ['label' => 'Tab 2', 'region_name' => 'tab_2'], 'weight' => 2],
    ];
  }

}
